/*
 * Copyright (C) 2023 Xeme Developers
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_MESSAGE_H
#define XEME_MESSAGE_H

#include <glib.h>
#include <glib-object.h>

#include "xemestanza.h"
#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeMessage:
 *
 * An XMPP Message stanza.
 *
 * Since: 0.1
 */

#define XEME_TYPE_MESSAGE (xeme_message_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeMessage, xeme_message, XEME, MESSAGE, XemeStanza)

/**
 * xeme_message_new:
 *
 * Creates a new [class@Message] instance.
 *
 * returns: (transfer full) (type XemeMessage): The new message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeStanza *xeme_message_new(void);

/**
 * xeme_message_get_thread:
 * @message: The instance.
 *
 * Gets the thread identifier of @message.
 *
 * Returns: (transfer none) (nullable): The thread id.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_message_get_thread(XemeMessage *message);

/**
 * xeme_message_set_thread:
 * @message: The instance.
 * @thread: (nullable): The new thread identifier.
 *
 * Sets the thread identifier of @message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_set_thread(XemeMessage *message, const char *thread);

/**
 * xeme_message_get_thread_parent:
 * @message: The instance.
 *
 * Gets the thread parent identifier of @message.
 *
 * Returns: (transfer none) (nullable): The thread id.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_message_get_thread_parent(XemeMessage *message);

/**
 * xeme_message_set_thread_parent:
 * @message: The instance.
 * @thread_parent: (nullable): The new thread parent identifier.
 *
 * Sets the thread parent identifier of @message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_set_thread_parent(XemeMessage *message, const char *thread_parent);

/**
 * xeme_message_add_subject:
 * @message: The instance.
 * @language: (nullable): The value for the xml:lang tag.
 * @content: The content.
 *
 * Add @content as a subject to @message. If a subject already exists with
 * @language, it will be replaced by the new @content.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_add_subject(XemeMessage *message, const char *language, const char *content);

/**
 * xeme_message_add_body:
 * @message: The instance.
 * @language: (nullable): The value for the xml:lang tag.
 * @content: The content.
 *
 * Add @content as a body to @message. If a body already exists with @language,
 * it will be replaced by the new @content.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_add_body(XemeMessage *message, const char *language, const char *content);

G_END_DECLS

#endif /* XEME_MESSAGE_H */
